/////////////////////////////////////////////////////////
//
//  EXONERAO DE RESPONSABILIDADE
//
// A Bolsa de Mercadorias & Futuros-BM&F fornece as informaes contidas neste cdigo fonte com propsito unicamente
// informativo e as disponibiliza apenas para atender s necessidades dos interessados em conhecer a tecnologia de referncia e
// suas convenincias. A Bolsa de Mercadorias & Futuros-BM&F no garante acurcia, integridade, perfeio ou ajuste a qualquer propsito
// especfico de tais informaes, tampouco aceita qualquer encargo, obrigao ou responsabilidade pelo uso das mesmas. As informaes
// disponibilizadas so obtidas de fontes entendidas como confiveis. No entanto, a Bolsa de Mercadorias & Futuros-BM&F no declara que as informaes
// disponibilizadas neste cdigo de referncia sejam verificadas, atualizadas e precisas. Devido  possibilidade de erro humano ou
// mecnico, bem como a outros fatores, a Bolsa de Mercadorias & Futuros-BM&F no responde por quaisquer erros ou omisses,
// dado que toda informao  provida "tal como est", sem nenhuma garantia de qualquer espcie. Nenhuma informao ou opinio
// aqui expressada constitui solicitao ou proposta de compra ou venda ou ainda oferta de compra ou de venda de qualquer dos
// contratos futuros e de opes negociados na Bolsa de Mercadorias & Futuros-BM&F, tampouco oferta de compra ou de venda de
// qualquer ttulo, valor mobilirio ou mercadoria aqui descrita como produto ou emisso. Como nenhuma informao disponvel
// neste cdigo de referncia deve servir de base para qualquer deciso de investimento, recomenda-se rigorosamente aos interessados que
// no confiem nem ajam fundamentados nas mesmas. Todavia, caso isso acontea, ser clara e inteiramente por sua conta e risco.
// Nem a Bolsa de Mercadorias & Futuros-BM&F nem seus executivos, diretores, associados, empregados, agentes, procuradores,
// consultores ou licenciados sero responsveis, perante qualquer pessoa (incluindo, mas no se limitando, um scio da Bolsa ou
// um cliente), por qualquer perda, dano, custo ou despesa (incluindo, mas no se limitando, lucros cessantes, impedimentos de uso
// ou danos diretos, indiretos, incidentais ou conseqentes) resultante de quaisquer erros, omisses ou alteraes nas informaes
// aqui fornecidas, tampouco por quaisquer atrasos, inexatides, erros, interrupes ou, ainda, quaisquer danos diretos ou
// conseqentes ocasionados ou mesmo ocorridos em funo do uso deste cdigo de referncia, nem, ainda, por qualquer prejuzo resultante de
// do seu mau uso, nem mesmo por qualquer descontinuidade do servio que venha a utiliz-lo.
// As disposies precedentes aplicam-se ainda que venha a surgir qualquer reivindicao ou pretenso de ordem contratual ou qualquer ao de
// reparao por ato ilcito extracontratual, negligncia, imprudncia, impercia, responsabilidade objetiva ou por qualquer outra maneira.
//
//
///////////////////////////////////////////////////////////
package br.com.bvmf.umdftcp.sample;
import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;


public class BranchInputStream extends FilterInputStream {
	
	public static final int EOF = -1;
    /**
     * The output stream that will receive a copy of all bytes read from the
     * proxied input stream.
     */
    private OutputStream branch = null;

    /**
     * Flag for closing also the associated output stream when this
     * stream is closed.
     */
    private boolean closeBranch = false;


    /**
     * Creates a BranchInputStream that proxies the given {@link InputStream}
     * and copies all read bytes to the given {@link OutputStream}. The given
     * output stream will not be closed when this stream gets closed.
     *
     * @param input input stream to be proxied
     * @param branch output stream that will receive a copy of all bytes read
     */
    protected BranchInputStream(final InputStream input, final OutputStream branch) {
		this(input, branch, false);
	}
     
    /**
     * Creates a BranchInputStream that proxies the given {@link InputStream}
     * and copies all read bytes to the given {@link OutputStream}. The given
     * output stream will be closed when this stream gets closed if the
     * closeBranch parameter is {@code true}.
     *
     * @param input input stream to be proxied
     * @param branch output stream that will receive a copy of all bytes read
     * @param closeBranch flag for closing also the output stream when this
     *                    stream is closed
     */
    public BranchInputStream(
            final InputStream input, final OutputStream branch, final boolean closeBranch) {
        super(input);
        this.branch = branch;
        this.closeBranch = closeBranch;
    }
    /**
     * Reads a single byte from the proxied input stream and writes it to
     * the associated output stream.
     *
     * @return next byte from the stream, or -1 if the stream has ended
     * @throws IOException if an I/O error occurs
     */
    public int read() throws IOException {
        try {
            final int b = in.read();
            if (b != EOF) {
                branch.write(b);
            }
            return b;
        } catch (IOException e) {
            handleIOException(e);
            return EOF;
        }
    }

    /**
     * Reads bytes from the proxied input stream and writes the read bytes
     * to the associated output stream.
     *
     * @param bts byte buffer
     * @return number of bytes read, or -1 if the stream has ended
     * @throws IOException if the stream could not be read (or written) 
     */
    public int read(byte[] bts) throws IOException {
        try {
            final int n = in.read(bts);
            if (n != EOF) {
                branch.write(bts, 0, n);
            }
            return n;
        } catch (IOException e) {
            handleIOException(e);
            return EOF;
        }
    }

    /**
     * Reads bytes from the proxied input stream and writes the read bytes
     * to the associated output stream.
     *
     * @param bts byte buffer
     * @param off offset within the buffer
     * @param len maximum number of bytes to read
     * @return number of bytes read, or -1 if the stream has ended
     * @throws IOException if the stream could not be read (or written) 
     */
    public int read(final byte[] bts, final int off, final int len) throws IOException {
        try {
            final int n = in.read(bts, off, len);
            if (n != EOF) {
                branch.write(bts, off, n);
            }
            return n;
        } catch (final IOException e) {
            handleIOException(e);
            return EOF;
        }
    }

    /**
     * Closes the proxied input stream and, if so configured, the associated
     * output stream. An exception thrown from one stream will not prevent
     * closing of the other stream.
     *
     * @throws IOException if either of the streams could not be closed
     */
    public void close() throws IOException {
        try {
            super.close();
        } finally {
            if (closeBranch) {
                branch.close();
            }
        }
    }
    
    protected void handleIOException(final IOException e) throws IOException {
        throw e;
    }
}
